const esbuild = require('esbuild');
const fs = require('fs');

function versionPlugin(version) {
  return {
    name: 'version',
    setup(build) {
      build.onLoad({ filter: /\.[jt]sx?$/ }, async (args) => {
        const contents = await fs.promises.readFile(args.path, 'utf8');
        return {
          contents: contents.replace(/__JOLIBOX_LOCAL_SDK_VERSION__/g, version),
          loader: args.path.endsWith('tsx') ? 'tsx' : 'ts'
        };
      });
    }
  };
}

function parseArgs() {
    const args = process.argv.slice(2);
    const options = {
      format: 'iife',  // 默认值
    };
  
    for (let i = 0; i < args.length; i++) {
      const arg = args[i];
      const [argKey, argVal] = arg.split('=');
      if (argKey === '--format' || argKey === '-f') {
        if (['esm', 'cjs', 'iife'].includes(argVal)) {
          options.format = argVal;
        }
        i++;
      }
    }
  
    return options;
  }
  
const options = parseArgs();

function build(format) {
    return esbuild.build({
        entryPoints: ['src/index.ts'],
        bundle: true,
        outfile: `dist/index.${format}.js`,
        target: ['es2015', 'safari14'],
        format: format,
        minify: true,
        loader: { 
          '.ts': 'ts',
          '.tsx': 'tsx',
          '.js': 'jsx'
        },
        plugins: [versionPlugin(process.env.BUILD_VERSION || '1.0.0')],
      }); 
}

(async ()=>{
    try{
        await build(options.format);
    }catch(e) {
        console.error(e);
        process.exit(1);
    }
})()

